using PizzeriaContracts.BindingModels;
using PizzeriaContracts.BusinessLogicsContracts;
using PizzeriaDataModels;
using Microsoft.Extensions.Logging;

namespace Pizzeria
{
    public partial class FormMain : Form
    {
        private readonly ILogger _logger;
        private readonly IOrderLogic _orderLogic;
        private readonly IReportLogic _reportLogic;
        public FormMain(ILogger<FormMain> logger, IOrderLogic orderLogic, IReportLogic reportLogic)
        {
            InitializeComponent();
            _logger = logger;
            _orderLogic = orderLogic;
            _reportLogic = reportLogic;
        }
        private void FormMain_Load(object sender, EventArgs e)
        {
            LoadData();
        }
        private void LoadData()
        {
            try
            {
                var list = _orderLogic.ReadList(null);
                if (list != null)
                {
                    dataGridView.DataSource = list;
                    dataGridView.Columns["PizzaId"].Visible = false;
                    dataGridView.Columns["ClientId"].Visible = false;
                    dataGridView.Columns["PizzaName"].AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill;
                }
                _logger.LogInformation(" ");
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "  ");
                MessageBox.Show(ex.Message, "", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        private void ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            var service = Program.ServiceProvider?.GetService(typeof(FormComponents));
            if (service is FormComponents form)
            {
                form.ShowDialog();
            }
        }
        private void ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            var service = Program.ServiceProvider?.GetService(typeof(FormPizzas));
            if (service is FormPizzas form)
            {
                form.ShowDialog();
            }
        }

        private void ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            var service = Program.ServiceProvider?.GetService(typeof(FormClients));
            if (service is FormClients form)
            {
                form.ShowDialog();
            }
        }
        private void ButtonCreateOrder_Click(object sender, EventArgs e)
        {
            var service = Program.ServiceProvider?.GetService(typeof(FormCreateOrder));
            if (service is FormCreateOrder form)
            {
                form.ShowDialog();
                LoadData();
            }
        }

        private OrderBindingModel getBindingModel()
        {
            return new OrderBindingModel
            {
                Id = Convert.ToInt32(dataGridView.SelectedRows[0].Cells["Id"].Value.ToString()),
                Count = Convert.ToInt32(dataGridView.SelectedRows[0].Cells["Count"].Value.ToString()),
                Sum = double.Parse(dataGridView.SelectedRows[0].Cells["Sum"].Value.ToString()),
                Status = Enum.Parse<OrderStatus>(dataGridView.SelectedRows[0].Cells["Status"].Value.ToString()),
                PizzaId = Convert.ToInt32(dataGridView.SelectedRows[0].Cells["PizzaId"].Value.ToString()),
                DateCreate = DateTime.SpecifyKind(DateTime.Parse(dataGridView.SelectedRows[0].Cells["DateCreate"].Value.ToString()), DateTimeKind.Utc),
            };
        }
        private void ButtonTakeOrderInWork_Click(object sender, EventArgs e)
        {
            if (dataGridView.SelectedRows.Count == 1)
            {
                int id = Convert.ToInt32(dataGridView.SelectedRows[0].Cells["Id"].Value);
                _logger.LogInformation(" {id}.    ' '", id);
            try
                {
                    var operationResult = _orderLogic.TakeOrderInWork(getBindingModel());
                    if (!operationResult)
                    {
                        throw new Exception("  .    .");
                    }
                    LoadData();
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "    ");
                    MessageBox.Show(ex.Message, "", MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
                }
            }
        }
        private void ButtonOrderReady_Click(object sender, EventArgs e)
        {
            if (dataGridView.SelectedRows.Count == 1)
            {
                int id =
               Convert.ToInt32(dataGridView.SelectedRows[0].Cells["Id"].Value);
                _logger.LogInformation(" {id}.    ''", id);
                try
                {
                    var operationResult = _orderLogic.FinishOrder(getBindingModel());
                    if (!operationResult)
                    {
                        throw new Exception("  .    .");
                    }
                    LoadData();
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "    ");
                MessageBox.Show(ex.Message, "", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }
        private void ButtonIssuedOrder_Click(object sender, EventArgs e)
        {
            if (dataGridView.SelectedRows.Count == 1)
            {
                int id = Convert.ToInt32(dataGridView.SelectedRows[0].Cells["Id"].Value);
                _logger.LogInformation(" {id}.    ''", id);
                try
                {
                    var operationResult = _orderLogic.DeliveryOrder(getBindingModel());
                    if (!operationResult)
                    {
                        throw new Exception("  .    .");
                    }
                    _logger.LogInformation(" {id} ", id);
                    LoadData();
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "    ");
                    MessageBox.Show(ex.Message, "", MessageBoxButtons.OK,
                   MessageBoxIcon.Error);
                }
            }
        }
        private void ButtonRef_Click(object sender, EventArgs e)
        {
            LoadData();
        }


        private void ComponentsToolStripMenuItem_Click(object sender, EventArgs
e)
        {
            using var dialog = new SaveFileDialog { Filter = "docx|*.docx" };
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                _reportLogic.SaveComponentsToWordFile(new ReportBindingModel
                {
                    FileName = dialog.FileName
                });
                MessageBox.Show("", "", MessageBoxButtons.OK,
               MessageBoxIcon.Information);
            }
        }
        private void ComponentPizzasToolStripMenuItem_Click(object sender, EventArgs e)
        {
            var service =
           Program.ServiceProvider?.GetService(typeof(FormReportPizzaComponents));
            if (service is FormReportPizzaComponents form)
            {
                form.ShowDialog();
            }
        }
        private void OrdersToolStripMenuItem_Click(object sender, EventArgs e)
        {
            var service = Program.ServiceProvider?.GetService(typeof(FormReportOrders));
            if (service is FormReportOrders form)
            {
                form.ShowDialog();
            }
        }
    }
}