﻿using PizzeriaContracts.BindingModels;
using PizzeriaContracts.SearchModels;
using PizzeriaContracts.StoragesContracts;
using PizzeriaContracts.ViewModels;
using PizzeriaDatabaseImplement.models;
using Microsoft.EntityFrameworkCore;


namespace PizzeriaDatabaseImplement.Implements
{
    public class OrderStorage : IOrderStorage
    {
        public List<OrderViewModel> GetFullList()
        {
            using var context = new PizzeriaDatabase();
            return context.Orders
                .Include(x => x.Pizza)
                .Include(x => x.Client)
                .Select(x => x.GetViewModel)
                .ToList();
        }
        public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
        {
            using var context = new PizzeriaDatabase();
            return context.Orders
                .Where(x => (
                        (!model.Id.HasValue || x.Id == model.Id) &&
                        (!model.DateFrom.HasValue || x.DateCreate >= model.DateFrom) &&
                        (!model.DateTo.HasValue || x.DateCreate <= model.DateTo)
                    )
                )
                .Select(x => x.GetViewModel)
                .ToList();
        }
        public OrderViewModel? GetElement(OrderSearchModel model)
        {
            using var context = new PizzeriaDatabase();
            return context.Orders
            .FirstOrDefault(x =>
                (model.Id.HasValue && x.Id == model.Id))?.GetViewModel;
        }
        public OrderViewModel? Insert(OrderBindingModel model)
        {
            using var context = new PizzeriaDatabase();
            Order newOrder = Order.Create(context, model);

            context.Orders.Add(newOrder);
            context.SaveChanges();
            return newOrder.GetViewModel;

        }
        public OrderViewModel? Update(OrderBindingModel model)
        {
            using var context = new PizzeriaDatabase();
            var order = context.Orders.FirstOrDefault(x => x.Id == model.Id);
            model.ClientId = order.ClientId;
            if (order == null)
            {
                return null;
            }
            order.Update(model);
            context.SaveChanges();
            return order.GetViewModel;
        }
        public OrderViewModel? Delete(OrderBindingModel model)
        {
            using var context = new PizzeriaDatabase();
            var element = context.Orders.FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                context.Orders.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }
    }

}
