﻿using PizzeriaContracts.BindingModels;
using PizzeriaContracts.ViewModels;
using PizzeriaDataModels.Models;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;

namespace PizzeriaDatabaseImplement.models
{
    public class Client : IClientModel
    {
        public int Id { get; private set; }
        [Required]
        public string ClientFIO { get; private set; }
        [Required]
        public string Email { get; private set; }
        [Required]
        public string Password { get; private set; }

        [ForeignKey("ClientId")]
        public virtual List<Order> Orders { get; set; } = new();
        public static Client? Create(ClientBindingModel model)
        {
            if (model == null)
            {
                return null;
            }
            return new Client()
            {
                Id = model.Id,
                ClientFIO = model.ClientFIO,
                Email = model.Email,
                Password = model.Password
            };
        }
        public static Client Create(ClientViewModel model)
        {
            return new Client
            {
                Id = model.Id,
                ClientFIO = model.ClientFIO,
                Email = model.Email,
                Password = model.Password
            };
        }
        public void Update(ClientBindingModel model)
        {
            if (model == null)
            {
                return;
            }
            Id = model.Id;
            ClientFIO = model.ClientFIO;
            Email = model.Email;
            Password = model.Password;
        }
        public ClientViewModel GetViewModel => new()
        {
            Id = Id,
            ClientFIO = ClientFIO,
            Email = Email,
            Password = Password
        };
    }
}
